package cn.com.ty.lift.business.common;

import cn.com.ty.lift.business.capital.dao.entity.CapitalRepair;
import cn.com.ty.lift.business.capital.dao.entity.CapitalRepairHistory;
import cn.com.ty.lift.business.capital.service.CapitalRepairHistoryService;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.LiftHistory;
import cn.com.ty.lift.business.library.service.LiftHistoryService;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.ProjectHistory;
import cn.com.ty.lift.business.project.service.ProjectHistoryService;
import cn.com.ty.lift.common.constants.FieldName;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author bieao
 * @date 2020/3/10
 * @description BeanUtil
 */
public class BeanUtils {

    @Resource
    private static ProjectHistoryService projectHistoryService;

    @Resource
    private static LiftHistoryService liftHistoryService;

    @Resource
    private static CapitalRepairHistoryService capitalRepairHistoryService;


    /**
     * @param oldEntry    旧实体
     * @param newEntry    新实体
     * @param mtCompanyId 公司id
     * @description 比较实体，并存入相应操作记录表
     * @date 2020/3/10 12:56 下午
     */
    public static void saveOperaHistory(Object oldEntry, Object newEntry, Long mtCompanyId) {
        Map<String, Object> oldResult = BeanUtil.beanToMap(oldEntry);
        Map<String, Object> newResult = BeanUtil.beanToMap(newEntry);
        List<ProjectHistory> projectHistories = new ArrayList<>();
        List<LiftHistory> liftHistories = new ArrayList<>();
        List<CapitalRepairHistory> capitalRepairHistories = new ArrayList<>();
        for (Map.Entry<String, Object> entry : oldResult.entrySet()) {
            Object oldValue = oldResult.get(entry.getKey());
            Object newValue = newResult.get(entry.getKey());
            boolean result = ObjectUtil.equal(oldValue, newValue);
            if (!result) {
                if (newEntry instanceof Project) {
                    Project project = (Project) newEntry;
                    Class<? extends Project> cls = project.getClass();
                    FieldName fieldName = cls.getDeclaredAnnotation(FieldName.class);
                    ProjectHistory projectHistory = new ProjectHistory();
                    projectHistory.setId(project.getId());
                    projectHistory.setMtCompanyId(mtCompanyId);
                    projectHistory.setOperaItem(fieldName.name());
                    projectHistory.setDescription("编辑项目信息");
                    projectHistory.setName(project.getProjectName());
                    projectHistory.setCode(project.getProjectCode());
                    projectHistory.setBeforeContent(oldValue.toString());
                    projectHistory.setAfterContent(newValue.toString());
                    projectHistory.setOperateDate(LocalDateTime.now());
                    projectHistories.add(projectHistory);
                } else if (newEntry instanceof Lift) {
                    Lift lift = (Lift) newEntry;
                    Class<? extends Lift> cls = lift.getClass();
                    FieldName fieldName = cls.getDeclaredAnnotation(FieldName.class);
                    LiftHistory liftHistory = new LiftHistory();
                    liftHistory.setId(lift.getId());
                    liftHistory.setCode(lift.getLiftCode());
                    liftHistory.setMtCompanyId(mtCompanyId);
                    liftHistory.setOperaItem(fieldName.name());
                    liftHistory.setDescription("编辑电梯信息");
                    liftHistory.setBeforeContent(oldValue.toString());
                    liftHistory.setAfterContent(newValue.toString());
                    liftHistory.setOperateDate(LocalDateTime.now());
                    liftHistories.add(liftHistory);
                } else if (newEntry instanceof CapitalRepair) {
                    CapitalRepair repair = (CapitalRepair) newEntry;
                    Class<? extends CapitalRepair> cls = repair.getClass();
                    FieldName fieldName = cls.getDeclaredAnnotation(FieldName.class);
                    CapitalRepairHistory capitalRepairHistory = new CapitalRepairHistory();
                    capitalRepairHistory.setId(repair.getId());
                    capitalRepairHistory.setProjectCode(repair.getProjectCode());
                    capitalRepairHistory.setMtCompanyId(mtCompanyId);
                    capitalRepairHistory.setOperaItem(fieldName.name());
                    capitalRepairHistory.setDescription("编辑大修信息");
                    capitalRepairHistory.setBeforeContent(oldValue.toString());
                    capitalRepairHistory.setAfterContent(newValue.toString());
                    capitalRepairHistory.setOperateDate(LocalDateTime.now());
                    capitalRepairHistories.add(capitalRepairHistory);
                }
            }
        }
        if (newEntry instanceof Project) {
            projectHistoryService.saveBatch(projectHistories, projectHistories.size());
        } else if (newEntry instanceof Lift) {
            liftHistoryService.saveBatch(liftHistories, liftHistories.size());
        } else if (newEntry instanceof CapitalRepair) {
            capitalRepairHistoryService.saveBatch(capitalRepairHistories, capitalRepairHistories.size());
        }
    }
}
